﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Inventor;

namespace SheetMetal
{
    public partial class Form1 : Form
    {
        private Inventor.Application mApp = null;

        public Form1(Inventor.Application oApp)
        {
            InitializeComponent();

            mApp = oApp;
        }

        public UnitVector GetFaceNormal(Face face)
        {
            SurfaceEvaluator oEvaluator = face.Evaluator;

            Double[] Points = { face.PointOnFace.X, face.PointOnFace.Y, face.PointOnFace.Z };

            Double[] GuessParams = new Double[2];
            Double[] MaxDev = new Double[2];
            Double[] Params = new Double[2];
            SolutionNatureEnum[] sol = new SolutionNatureEnum[2];

            oEvaluator.GetParamAtPoint(ref Points, ref GuessParams, ref MaxDev, ref Params, ref sol);

            Double[] normal = new Double[3];

            oEvaluator.GetNormal(ref Params, ref normal);

            return mApp.TransientGeometry.CreateUnitVector(normal[0], normal[1], normal[2]);
        }

        private PartDocument CreateSheetMetal()
        {
            // 1. Declare a variable as a PartDocument. Instantiate it by using the 
            // Add method of the Document collection of the Applicaiton. 
            // for the document type using kPartDocumentObject of the DocumentTypeEnum.
            // for the TemplateFileName argument use GetTemplateFile of the FileManager
            // property of the Inventor Application. For the document type use kPartDocumentObject
            // of the DocumentTypeEnum. For the SystemOfMeasure use kDefaultSystemOfMeasure
            // of the SystemOfMeasureEnum. For the DraftingStandard use kDefault_DraftingStandard
            // of the DraftingStandardEnum. 
            // For the DocumentSubType use "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}"
            PartDocument oSheetMetalDoc = mApp.Documents.Add(DocumentTypeEnum.kPartDocumentObject,
                mApp.FileManager.GetTemplateFile(DocumentTypeEnum.kPartDocumentObject,
                    SystemOfMeasureEnum.kDefaultSystemOfMeasure, 
                    DraftingStandardEnum.kDefault_DraftingStandard,
                    "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}"),
                true) as PartDocument;
                                  
            //2. Declare a varialbe as a SheetMetalComponentDefinition. Instantiate it by
            //using the ComponentDefinition property of the PartDocument created in step 1      
            SheetMetalComponentDefinition oCompDef = oSheetMetalDoc.ComponentDefinition as SheetMetalComponentDefinition; 
           
            //3. Declare a SheetMetalFeatures variable and make it equal to the Features
            //collection of the SheetMetalComponentDefinition created in step 2.
            SheetMetalFeatures oSheetMetalFeatures = oCompDef.Features as SheetMetalFeatures;

            // 4. Declare a PlanarSketch variable and instantiate it by using the Add
            // method of the Sketches collecion of the SheetMetalComponentDefinition
            // created in step 2. For the Planar Entity argument use the Item property
            // of the WorkPlanes colleciton of the SheetMetalComponentDefinition. Use
            // 3 for the Index argument. (X-Y work plane)
            PlanarSketch oSketch = oCompDef.Sketches.Add(oCompDef.WorkPlanes[3], false);

            //5. Declare a varialbe as TransientGeometry and make it equal to the 
            //TransientGeometry property of the Application. (_InvApplication)
            TransientGeometry oTransGeom = mApp.TransientGeometry;

            //6. Draw a 20cm x 15cm rectangle with the corner at (0,0) by using 
            //AddAsTwoPointRectangle of the SketchLines collection of the sketch 
            //created in step 4. Use the TransientGeometry created in step 5 to
            //create Point2d object for the corner points.
            oSketch.SketchLines.AddAsTwoPointRectangle(oTransGeom.CreatePoint2d(0, 0),
                oTransGeom.CreatePoint2d(20, 15));

            //7. Declare a variable as a profile instantiate it using the AddForSolid
            //method of the Profiles collection of the Sketch created in step 4
            Profile oProfile = oSketch.Profiles.AddForSolid(true, null, null);
        
            //8. Declare a variable as a FaceFeatureDefinition. Instantiate it using the 
            //CreateFaceFeatureDefinition of the FaceFeatures collection of the 
            //SheetMetalComponentDefinition created in step 2. Use the Profile 
            //created in step 7 for the argument.
            FaceFeatureDefinition oFaceFeatureDefinition = oSheetMetalFeatures.FaceFeatures.CreateFaceFeatureDefinition(oProfile); 
       
            //9. Declare a FaceFeature variable. Instantiate it using the Add method
            //of the FaceFeatures collection of the SheetMetalComponentDefinition created in 
            //step 2. Pass in the FaceFeatureDefinition created in step 8.
            FaceFeature oFaceFeature = oSheetMetalFeatures.FaceFeatures.Add(oFaceFeatureDefinition);
      
            //10. Create a UnitVector used to find which face is the top most
            UnitVector zAxis = oTransGeom.CreateUnitVector(0,0,1);

            Face oFrontFace = null;

            foreach (Face face in oCompDef.SurfaceBodies[1].Faces)
            {
                if (System.Math.Abs(GetFaceNormal(face).AngleTo(zAxis)) < 0.001)
                {
                    oFrontFace = face;
                    break;
                }
            }

            //11. Create a new sketch on the face created in step 12. Make the Sketch variable
            //declared in step 4 equal to the return of the Add method of the Sketches collecition of the 
            //SheetMetalComponentDefinition created in 'step 2. Pass in the face retrieved in step 10.
            oSketch = oCompDef.Sketches.Add(oFrontFace, false);

            //12. Draw a 3cm x 5.5cm rectangle with the corners at (2,5.5) and (5,11) by using 
            //AddAsTwoPointRectangle of the SketchLines collection of the sketch 
            //instantiated in step 13. (this will be a cut) Use the TransientGeometry created 
            //in step 5 to create Point2d objects for the corner points.
            oSketch.SketchLines.AddAsTwoPointRectangle(oTransGeom.CreatePoint2d(2, 5.5), oTransGeom.CreatePoint2d(5, 11));

            //13. Use the profile declared in step 7. Instantiate it using the AddForSolid
            //method of the Profiles collection of the Sketch instantiated in step 13.
            oProfile = oSketch.Profiles.AddForSolid(false, null, null);

            //14. Declare a CutDefinition object. Instantiate it by using the CreateCutDefinition
            //of the CutFeatures collection of the SheetMetalComponentDefinition created in 'step 2
            //Pass in the profile from step 13.
            CutDefinition oCutDefinition = oSheetMetalFeatures.CutFeatures.CreateCutDefinition(oProfile);
          
            //15. Set extents to 'Through All' by using the SetThroughAllExtent method of
            //the CutDefinition object created in step 14. For the direction argument use
            //kNegativeExtentDirection of PartFeatureExtentDirectionEnum. 
            oCutDefinition.SetThroughAllExtent(PartFeatureExtentDirectionEnum.kNegativeExtentDirection);

            //16. Declare a CutFeature variable. Instantiate it using the Add method
            //of the CutFeatures collection of the SheetMetalComponentDefinition created in 'step 2
            //Pass in the CutDefinition created in step 14.
            CutFeature oCutFeature = oSheetMetalFeatures.CutFeatures.Add(oCutDefinition);
         
            //17. Create a new sketch on the X-Z work plane.
            //Make the Sketch variable declared in step 4 equal to the return of the
            //Add method of the Sketches collecition of the 
            //SheetMetalComponentDefinition created in 'step 2. Use the WorkPlanes collection
            //(Item 2)
            oSketch = oCompDef.Sketches.Add(oCompDef.WorkPlanes[2], false);

            //18. Draw a 15cm x 10cm rectangle with the corner at (0,0)
            //Use the AddAsTwoPointRectangle. Use TransientGeometry created in step 5
            oSketch.SketchLines.AddAsTwoPointRectangle(oTransGeom.CreatePoint2d(0, 0), oTransGeom.CreatePoint2d(-15, 10));
                       
            //19. Use the profile declared in step 7. Instantiate it using the AddForSolid
            //method of the Profiles collection of the Sketch instantiated in step 17.
            oProfile = oSketch.Profiles.AddForSolid(false, null, null);

            //20. Use the FaceFeatureDefinition variable created in step 8. Make it equal to 
            //the return of the CreateFaceFeatureDefinition method of the FaceFeatures collection
            //of the SheetMetalComponentDefinition created in 'step 2. Pass in the profile
            //created in step 19.
            oFaceFeatureDefinition = oSheetMetalFeatures.FaceFeatures.CreateFaceFeatureDefinition(oProfile);

            //21. USe the FaceFeature variable and make it equal to the return of the Add
            //method of the FaceFeatures collection of the SheetMetalComponentDefinition.
            //pass in the FaceFeatureDefinition instantiated in step 22.
            oFaceFeature = oSheetMetalFeatures.FaceFeatures.Add(oFaceFeatureDefinition);

            return oSheetMetalDoc;
        }

        private void WriteSheetMetalDXF(PartDocument oPartDoc)
        {
            if( oPartDoc.SubType != "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}")
            {
                //Doc must be SheetMetal Part
                return;
            }

            DataIO oDataIO = oPartDoc.ComponentDefinition.DataIO;
           
            string sOut = "FLAT PATTERN DXF?AcadVersion=R12&OuterProfileLayer=Outer";
            
            oDataIO.WriteDataToFile(sOut, @"C:\Temp\flat.dxf");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            PartDocument oPartDoc = CreateSheetMetal();

            SheetMetalComponentDefinition oCompDef= oPartDoc.ComponentDefinition as SheetMetalComponentDefinition;

            oCompDef.Unfold();

            WriteSheetMetalDXF(oPartDoc);
        }
    }
}
