﻿Imports Inventor
Imports System.Reflection

Public Class Form1

    Private _InvApplication As Inventor.Application
  
    Public Sub New(ByVal oApp As Inventor.Application)

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        _InvApplication = oApp

    End Sub

  
    '/////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '// Use:
    '// This sample demonstrates the creation of sheet metal face and cut features.
    '// It creates a new sheet metal document, create a face feature, a cut feature
    '// and another face feature.  The second face feature butts up to the first
    '// face feature so it automatically creates a bend between them.
    '//
    '/////////////////////////////////////////////////////////////////////////////////////////////////////////////
    Private Function CreateSheetMetal()

        ' 1. Declare a variable as a PartDocument. Instantiate it by using the 
        ' Add method of the Document collection of the Applicaiton. (_InvApplication)
        ' for the document type using kPartDocumentObject of the DocumentTypeEnum.
        ' for the TemplateFileName argument use GetTemplateFile of the FileManager
        ' property of the Inventor Application. For the document type use kPartDocumentObject
        ' of the DocumentTypeEnum. For the SystemOfMeasure use kDefaultSystemOfMeasure
        ' of the SystemOfMeasureEnum. For the DraftingStandard use kDefault_DraftingStandard
        ' of the DraftingStandardEnum. 
        ' For the DocumentSubType use "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}"
        Dim oSheetMetalDoc As PartDocument
        oSheetMetalDoc = _InvApplication.Documents.Add(DocumentTypeEnum.kPartDocumentObject, _
                     _InvApplication.FileManager.GetTemplateFile(DocumentTypeEnum.kPartDocumentObject, _
                                                                 SystemOfMeasureEnum.kDefaultSystemOfMeasure, _
                                                                 DraftingStandardEnum.kDefault_DraftingStandard, _
                                                                 "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}"))

        ' 2. Declare a varialbe as a SheetMetalComponentDefinition. Instantiate it by
        ' using the ComponentDefinition property of the PartDocument created in step 1.
        Dim oCompDef As SheetMetalComponentDefinition
        oCompDef = oSheetMetalDoc.ComponentDefinition

        ' 3. Declare a SheetMetalFeatures variable and make it equal to the Features
        ' collection of the SheetMetalComponentDefinition created in step 2.
        Dim oSheetMetalFeatures As SheetMetalFeatures
        oSheetMetalFeatures = oCompDef.Features

        ' 4. Declare a PlanarSketch variable and instantiate it by using the Add
        ' method of the Sketches collecion of the SheetMetalComponentDefinition
        ' created in step 2. For the Planar Entity argument use the Item property
        ' of the WorkPlanes colleciton of the SheetMetalComponentDefinition. Use
        ' 3 for the Index argument. (X-Y work plane)
        Dim oSketch As PlanarSketch
        oSketch = oCompDef.Sketches.Add(oCompDef.WorkPlanes.Item(3))

        ' 5. Declare a varialbe as TransientGeometry and make it equal to the 
        ' TransientGeometry property of the Application. (_InvApplication)
        Dim oTransGeom As TransientGeometry
        oTransGeom = _InvApplication.TransientGeometry

        ' 6. Draw a 20cm x 15cm rectangle with the corner at (0,0) by using 
        ' AddAsTwoPointRectangle of the SketchLines collection of the sketch 
        ' created in step 4. Use the TransientGeometry created in step 5 to
        ' create Point2d object for the corner points.
        Call oSketch.SketchLines.AddAsTwoPointRectangle( _
                                    oTransGeom.CreatePoint2d(0, 0), _
                                    oTransGeom.CreatePoint2d(20, 15))

        ' 7. Declare a variable as a profile instantiate it using the AddForSolid
        ' method of the Profiles collection of the Sketch created in step 4
        Dim oProfile As Profile
        oProfile = oSketch.Profiles.AddForSolid

        ' 8. Declare a variable as a FaceFeatureDefinition. Instantiate it using the 
        ' CreateFaceFeatureDefinition of the FaceFeatures collection of the 
        ' SheetMetalComponentDefinition created in step 2. Use the Profile 
        ' created in step 7 for the argument.
        Dim oFaceFeatureDefinition As FaceFeatureDefinition
        oFaceFeatureDefinition = oSheetMetalFeatures.FaceFeatures.CreateFaceFeatureDefinition(oProfile)

        ' 9. Declare a FaceFeature variable. Instantiate it using the Add method
        ' of the FaceFeatures collection of the SheetMetalComponentDefinition created in 
        'step 2. Pass in the FaceFeatureDefinition created in step 8.
        Dim oFaceFeature As FaceFeature
        oFaceFeature = oSheetMetalFeatures.FaceFeatures.Add(oFaceFeatureDefinition)

        ' 10. Declare a zero length array as SelectionFilterEnum. Instantitate the first element
        ' using kPartFaceFilter of the SelectionFilterEnum
        Dim aSelectTypes(0) As SelectionFilterEnum
        aSelectTypes(0) = SelectionFilterEnum.kPartFaceFilter

        ' 11. Declare a vaiable as an ObjectsEnumerator. Name it something like
        ' oFoundFaces. Instantiate it using the FindUsingPoint method of the 
        ' SheetMetalComponentDefinition created in step 2. For the Point argument use
        ' the CreatePoint method of the TransientGeometry object created in step 5.
        ' Use 1,1 for X and Y. For Z use the Value property of the Thickness property
        ' of the SheetMetalComponentDefinition. For the third argument pass in
        ' the SelectionFilterEnum created in step 10. For the Proximity Tolerance use .001
        Dim oFoundFaces As ObjectsEnumerator
        oFoundFaces = oCompDef.FindUsingPoint(oTransGeom.CreatePoint(1, 1, oCompDef.Thickness.Value), aSelectTypes, 0.001)


        ' 12. Declare a Face vaiable named oFrontFace. Instantiate it by making it 
        ' equal to the first item in the ObjectsEnumerator created in step 11.
        Dim oFrontFace As Face
        oFrontFace = oFoundFaces.Item(1)

        ' 13. Create a new sketch on the face created in step 12. Make the Sketch variable
        ' declared in step 4 equal to the return of the Add method of the Sketches collecition of the 
        ' SheetMetalComponentDefinition created in 'step 2. Pass in the face created in step 12.
        oSketch = oCompDef.Sketches.Add(oFrontFace)

        ' 14. Draw a 3cm x 5.5cm rectangle with the corners at (2,5.5) and (5,11) by using 
        ' AddAsTwoPointRectangle of the SketchLines collection of the sketch 
        ' instantiated in step 13. (this will be a cut) Use the TransientGeometry created 
        ' in step 5 to create Point2d objects for the corner points.
        Call oSketch.SketchLines.AddAsTwoPointRectangle( _
                    oTransGeom.CreatePoint2d(2, 5.5), _
                    oTransGeom.CreatePoint2d(5, 11))

        ' 15. Use the profile declared in step 7. Instantiate it using the AddForSolid
        ' method of the Profiles collection of the Sketch instantiated in step 13.
        oProfile = oSketch.Profiles.AddForSolid

        ' 16. Declare a CutDefinition object. Instantiate it by using the CreateCutDefinition
        ' of the CutFeatures collection of the SheetMetalComponentDefinition created in 'step 2
        ' Pass in the profile from step 15.
        Dim oCutDefinition As CutDefinition
        oCutDefinition = oSheetMetalFeatures.CutFeatures.CreateCutDefinition(oProfile)

        ' 17. Set extents to 'Through All' by using the SetThroughAllExtent method of
        ' the CutDefinition object created in step 16. For the direction argument use
        ' kNegativeExtentDirection of PartFeatureExtentDirectionEnum. 
        Call oCutDefinition.SetThroughAllExtent(PartFeatureExtentDirectionEnum.kNegativeExtentDirection)

        ' 18. Declare a CutFeature variable. Instantiate it using the Add method
        ' of the CutFeatures collection of the SheetMetalComponentDefinition created in 'step 2
        ' Pass in the CutDefinition created in step 16.
        Dim oCutFeature As CutFeature
        oCutFeature = oSheetMetalFeatures.CutFeatures.Add(oCutDefinition)

        ' 19. Create a new sketch on the X-Z work plane.
        ' Make the Sketch variable declared in step 4 equal to the return of the
        ' Add method of the Sketches collecition of the 
        ' SheetMetalComponentDefinition created in 'step 2. Use the WorkPlanes collection
        ' (Item 2)
        oSketch = oCompDef.Sketches.Add(oCompDef.WorkPlanes.Item(2))

        ' 20. Draw a 15cm x 10cm rectangle with the corner at (0,0)
        ' Use the AddAsTwoPointRectangle. Use TransientGeometry created in step 5
        Call oSketch.SketchLines.AddAsTwoPointRectangle( _
                                    oTransGeom.CreatePoint2d(0, 0), _
                                    oTransGeom.CreatePoint2d(-15, 10))

        ' 21. Use the profile declared in step 7. Instantiate it using the AddForSolid
        ' method of the Profiles collection of the Sketch instantiated in step 19.
        oProfile = oSketch.Profiles.AddForSolid

        '22. Use the FaceFeatureDefinition variable created in step 8. Make it equal to 
        ' the return of the CreateFaceFeatureDefinition method of the FaceFeatures collection
        ' of the SheetMetalComponentDefinition created in 'step 2. Pass in the profile
        ' created in step 21.
        oFaceFeatureDefinition = oSheetMetalFeatures.FaceFeatures.CreateFaceFeatureDefinition(oProfile)

        ' 23. USe the FaceFeature variable and make it equal to the return of the Add
        ' method of the FaceFeatures collection of the SheetMetalComponentDefinition.
        ' pass in the FaceFeatureDefinition instantiated in step 22.
        oFaceFeature = oSheetMetalFeatures.FaceFeatures.Add(oFaceFeatureDefinition)

        Return oSheetMetalDoc

    End Function

    '/////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '// Use:
    '// demonstrates creating an R12 DXF file that will have a layer called "Outer"
    '// where the curves for the outer shape will be created.
    '/////////////////////////////////////////////////////////////////////////////////////////////////////////////
    Private Sub WriteSheetMetalDXF(ByVal oPartDoc As PartDocument)

        ' 2. Use an "If" statment and Make sure the document is a sheet metal document.
        ' use the SubType property of PartDocument created in step 1.
        ' As a test use "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}" If the 
        ' document is not a SheetMetal document use a MsgBox to inform the
        ' user and Exit the sub.
        If oPartDoc.SubType <> "{9C464203-9BAE-11D3-8BAD-0060B0CE6BB4}" Then
            Exit Sub
        End If

        ' 3. Declare a variable as a DataIO object. Instantiate it using the 
        ' Get the DataIO property of the ComponentDefinition property of the 
        ' PartDocument created in step 1.
        Dim oDataIO As DataIO
        oDataIO = oPartDoc.ComponentDefinition.DataIO

        ' 4. Build the string that defines the format of the DXF file.
        ' Declare a String variable. Make it equal to the following:
        ' "FLAT PATTERN DXF?AcadVersion=R12&OuterProfileLayer=Outer"
        Dim sOut As String
        sOut = "FLAT PATTERN DXF?AcadVersion=R12&OuterProfileLayer=Outer"

        ' 5. Create the DXF file by calling the WriteDataToFile method
        ' of the DataIO object created in step 3. For the first argument
        ' (format string) pass in the string created in step 4. For the
        ' FileName arguement use something like "c:\Temp\flat.dxf"
        oDataIO.WriteDataToFile(sOut, "c:\Temp\flat.dxf")

    End Sub


    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click

        Dim oPartDoc As PartDocument = CreateSheetMetal()

        Dim oCompDef As SheetMetalComponentDefinition = oPartDoc.ComponentDefinition

        oCompDef.Unfold()

        WriteSheetMetalDXF(oPartDoc)

    End Sub

End Class
